package java.awt.font;

import java.awt.Font;
import java.util.HashMap;
import java.awt.Toolkit;
import java.awt.im.InputMethodHighlight;
import java.text.Annotation;
import sun.text.CodePointIterator;
import sun.font.FontResolver;
import java.util.Map;
import java.text.AttributedCharacterIterator;
import java.util.Vector;
import sun.font.Decoration;

final class StyledParagraph
{
    private int length;
    private Decoration decoration;
    private Object font;
    private Vector<Decoration> decorations;
    int[] decorationStarts;
    private Vector<Object> fonts;
    int[] fontStarts;
    private static int INITIAL_SIZE;
    
    public StyledParagraph(AttributedCharacterIterator attributedCharacterIterator, char[] array) {
        int i = attributedCharacterIterator.getBeginIndex();
        int endIndex = attributedCharacterIterator.getEndIndex();
        this.length = endIndex - i;
        attributedCharacterIterator.first();
        do {
            int runLimit = attributedCharacterIterator.getRunLimit();
            int n = i - i;
            Map<Object, Object> addInputMethodAttrs = addInputMethodAttrs(attributedCharacterIterator.getAttributes());
            this.addDecoration(Decoration.getDecoration(addInputMethodAttrs), n);
            Object graphicOrFont = getGraphicOrFont(addInputMethodAttrs);
            if (graphicOrFont == null) {
                this.addFonts(array, addInputMethodAttrs, n, runLimit - i);
            }
            else {
                this.addFont(graphicOrFont, n);
            }
            attributedCharacterIterator.setIndex(runLimit);
            i = runLimit;
        } while (i < endIndex);
        if (this.decorations != null) {
            this.decorationStarts = addToVector(this, this.length, this.decorations, this.decorationStarts);
        }
        if (this.fonts != null) {
            this.fontStarts = addToVector(this, this.length, this.fonts, this.fontStarts);
        }
    }
    
    private static void insertInto(int n, int[] array, int n2) {
        while (array[--n2] > n) {
            int n3 = n2;
            ++array[n3];
        }
    }
    
    public static StyledParagraph insertChar(AttributedCharacterIterator attributedCharacterIterator, char[] array, int index, StyledParagraph styledParagraph) {
        int setIndex = attributedCharacterIterator.setIndex(index);
        int max = Math.max(index - attributedCharacterIterator.getBeginIndex() - 1, 0);
        Map<Object, Object> addInputMethodAttrs = addInputMethodAttrs(attributedCharacterIterator.getAttributes());
        if (!styledParagraph.getDecorationAt(max).equals(Decoration.getDecoration(addInputMethodAttrs))) {
            return new StyledParagraph(attributedCharacterIterator, array);
        }
        Object obj = getGraphicOrFont(addInputMethodAttrs);
        if (obj == null) {
            FontResolver instance = FontResolver.getInstance();
            obj = instance.getFont(instance.getFontIndex((char)setIndex), addInputMethodAttrs);
        }
        if (!styledParagraph.getFontOrGraphicAt(max).equals(obj)) {
            return new StyledParagraph(attributedCharacterIterator, array);
        }
        ++styledParagraph.length;
        if (styledParagraph.decorations != null) {
            insertInto(max, styledParagraph.decorationStarts, styledParagraph.decorations.size());
        }
        if (styledParagraph.fonts != null) {
            insertInto(max, styledParagraph.fontStarts, styledParagraph.fonts.size());
            return styledParagraph;
        }
        return styledParagraph;
    }
    
    private static void deleteFrom(int n, int[] array, int n2) {
        while (array[--n2] > n) {
            int n3 = n2;
            --array[n3];
        }
    }
    
    public static StyledParagraph deleteChar(AttributedCharacterIterator attributedCharacterIterator, char[] array, int n, StyledParagraph styledParagraph) {
        n -= attributedCharacterIterator.getBeginIndex();
        if (styledParagraph.decorations == null && styledParagraph.fonts == null) {
            --styledParagraph.length;
            return styledParagraph;
        }
        if (styledParagraph.getRunLimit(n) == n + 1 && (n == 0 || styledParagraph.getRunLimit(n - 1) == n)) {
            return new StyledParagraph(attributedCharacterIterator, array);
        }
        --styledParagraph.length;
        if (styledParagraph.decorations != null) {
            deleteFrom(n, styledParagraph.decorationStarts, styledParagraph.decorations.size());
        }
        if (styledParagraph.fonts != null) {
            deleteFrom(n, styledParagraph.fontStarts, styledParagraph.fonts.size());
            return styledParagraph;
        }
        return styledParagraph;
    }
    
    public int getRunLimit(int n) {
        if (n < 0 || n >= this.length) {
            throw new IllegalArgumentException("index out of range");
        }
        int length = this.length;
        if (this.decorations != null) {
            length = this.decorationStarts[findRunContaining(n, this.decorationStarts) + 1];
        }
        int length2 = this.length;
        if (this.fonts != null) {
            length2 = this.fontStarts[findRunContaining(n, this.fontStarts) + 1];
        }
        return Math.min(length, length2);
    }
    
    public Decoration getDecorationAt(int n) {
        if (n < 0 || n >= this.length) {
            throw new IllegalArgumentException("index out of range");
        }
        if (this.decorations == null) {
            return this.decoration;
        }
        return (Decoration)this.decorations.elementAt(findRunContaining(n, this.decorationStarts));
    }
    
    public Object getFontOrGraphicAt(int n) {
        if (n < 0 || n >= this.length) {
            throw new IllegalArgumentException("index out of range");
        }
        if (this.fonts == null) {
            return this.font;
        }
        return this.fonts.elementAt(findRunContaining(n, this.fontStarts));
    }
    
    private static int findRunContaining(int n, int[] array) {
        int n2;
        for (n2 = 1; array[n2] <= n; ++n2) {}
        return n2 - 1;
    }
    
    private static int[] addToVector(Object o, int n, Vector vector, int[] array) {
        if (!vector.lastElement().equals(o)) {
            vector.addElement(o);
            int size = vector.size();
            if (array.length == size) {
                int[] array2 = new int[array.length * 2];
                System.arraycopy(array, 0, array2, 0, array.length);
                array = array2;
            }
            array[size - 1] = n;
            return array;
        }
        else {
            return array;
        }
    }
    
    private void addDecoration(Decoration decoration, int n) {
        if (this.decorations != null) {
            this.decorationStarts = addToVector(decoration, n, this.decorations, this.decorationStarts);
        }
        else if (this.decoration == null) {
            this.decoration = decoration;
        }
        else if (!this.decoration.equals(decoration)) {
            (this.decorations = new Vector(StyledParagraph.INITIAL_SIZE)).addElement(this.decoration);
            this.decorations.addElement(decoration);
            (this.decorationStarts = new int[StyledParagraph.INITIAL_SIZE])[0] = 0;
            this.decorationStarts[1] = n;
        }
    }
    
    private void addFont(Object font, int n) {
        if (this.fonts != null) {
            this.fontStarts = addToVector(font, n, this.fonts, this.fontStarts);
        }
        else if (this.font == null) {
            this.font = font;
        }
        else if (!this.font.equals(font)) {
            (this.fonts = new Vector(StyledParagraph.INITIAL_SIZE)).addElement(this.font);
            this.fonts.addElement(font);
            (this.fontStarts = new int[StyledParagraph.INITIAL_SIZE])[0] = 0;
            this.fontStarts[1] = n;
        }
    }
    
    private void addFonts(char[] array, Map<? extends AttributedCharacterIterator.Attribute, ?> attributes, int n, int n2) {
        FontResolver instance = FontResolver.getInstance();
        CodePointIterator create = CodePointIterator.create(array, n, n2);
        for (int i = create.charIndex(); i < n2; i = create.charIndex()) {
            this.addFont(instance.getFont(instance.nextFontRunIndex(create), attributes), i);
        }
    }
    
    static Map<? extends AttributedCharacterIterator.Attribute, ?> addInputMethodAttrs(Map<? extends AttributedCharacterIterator.Attribute, ?> m) {
        Object o = m.get(TextAttribute.INPUT_METHOD_HIGHLIGHT);
        try {
            if (o != null) {
                if ((o instanceof Annotation)) {
                    o = ((Annotation)o).getValue();
                }
                InputMethodHighlight inputMethodHighlight = (InputMethodHighlight)o;
                Map<TextAttribute, Object> i = null;
                try {
                    i = inputMethodHighlight.getStyle();
                }
                catch (NoSuchMethodError noSuchMethodError) {}
                if (i == null) {
                    i = Toolkit.getDefaultToolkit().mapInputMethodHighlight(inputMethodHighlight);
                }
                if (i != null) {
                    HashMap o2 = new HashMap(5, 0.9f);
                    o2.putAll(m);
                    o2.putAll(i);
                    return o2;
                }
            }
        }
        catch (ClassCastException ex) {}
        return m;
    }
    
    private static Object getGraphicOrFont(Map<? extends AttributedCharacterIterator.Attribute, ?> attributes) {
        Object value = attributes.get(TextAttribute.CHAR_REPLACEMENT);
        if (value != null) {
            return value;
        }
        Object value2 = attributes.get(TextAttribute.FONT);
        if (value2 != null) {
            return value2;
        }
        if (attributes.get(TextAttribute.FAMILY) != null) {
            return Font.getFont(attributes);
        }
        return null;
    }
    
    static {
        StyledParagraph.INITIAL_SIZE = 8;
    }
}
